package com.mdd.admin.service.impl;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONReader;
import com.alibaba.fastjson2.JSONWriter;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.mdd.admin.service.ISystemAuthDeptService;
import com.mdd.admin.validate.system.SystemDeptCreateValidate;
import com.mdd.admin.validate.system.SystemDeptSearchValidate;
import com.mdd.admin.validate.system.SystemDeptUpdateValidate;
import com.mdd.admin.vo.system.SystemAuthDeptVo;
import com.mdd.common.entity.system.SystemAuthAdmin;
import com.mdd.common.entity.system.SystemAuthDept;
import com.mdd.common.mapper.system.SystemAuthAdminMapper;
import com.mdd.common.mapper.system.SystemAuthDeptMapper;
import com.mdd.common.util.ListUtils;
import com.mdd.common.util.TimeUtils;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import javax.annotation.Resource;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

@Service
class SystemAuthDeptServiceImpl implements ISystemAuthDeptService {
   @Resource
   SystemAuthDeptMapper systemAuthDeptMapper;
   @Resource
   SystemAuthAdminMapper systemAuthAdminMapper;

   public List<SystemAuthDeptVo> all() {
      List<SystemAuthDept> systemAuthDeptList = this.systemAuthDeptMapper.selectList((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).gt("pid", 0)).eq("is_delete", 0)).orderByDesc(Arrays.asList("sort", "id")));
      List<SystemAuthDeptVo> list = new ArrayList();
      Iterator var3 = systemAuthDeptList.iterator();

      while(var3.hasNext()) {
         SystemAuthDept systemAuthDept = (SystemAuthDept)var3.next();
         SystemAuthDeptVo vo = new SystemAuthDeptVo();
         BeanUtils.copyProperties(systemAuthDept, vo);
         vo.setUpdateTime(TimeUtils.timestampToDate(systemAuthDept.getUpdateTime()));
         vo.setCreateTime(TimeUtils.timestampToDate(systemAuthDept.getCreateTime()));
         list.add(vo);
      }

      return list;
   }

   public JSONArray list(SystemDeptSearchValidate searchValidate) {
      QueryWrapper<SystemAuthDept> queryWrapper = new QueryWrapper();
      queryWrapper.eq("is_delete", 0);
      queryWrapper.orderByDesc(Arrays.asList("sort", "id"));
      queryWrapper.select(SystemAuthDept.class, (info) -> {
         return !info.getColumn().equals("is_delete") && !info.getColumn().equals("delete_time");
      });
      this.systemAuthDeptMapper.setSearch(queryWrapper, searchValidate, new String[]{"like:name:str", "=:isStop@is_stop:int"});
      List<SystemAuthDept> systemAuthDeptList = this.systemAuthDeptMapper.selectList(queryWrapper);
      List<SystemAuthDeptVo> list = new LinkedList();
      Iterator var5 = systemAuthDeptList.iterator();

      while(var5.hasNext()) {
         SystemAuthDept systemAuthDept = (SystemAuthDept)var5.next();
         SystemAuthDeptVo vo = new SystemAuthDeptVo();
         BeanUtils.copyProperties(systemAuthDept, vo);
         vo.setCreateTime(TimeUtils.timestampToDate(systemAuthDept.getCreateTime()));
         vo.setUpdateTime(TimeUtils.timestampToDate(systemAuthDept.getUpdateTime()));
         list.add(vo);
      }

      JSONArray jsonArray = JSONArray.parseArray(JSONArray.toJSONString(list, new JSONWriter.Feature[0]), new JSONReader.Feature[0]);
      return ListUtils.listToTree(jsonArray, "id", "pid", "children");
   }

   public SystemAuthDeptVo detail(Integer id) {
      SystemAuthDept systemAuthDept = (SystemAuthDept)this.systemAuthDeptMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(SystemAuthDept.class, (info) -> {
         return !info.getColumn().equals("is_delete") && !info.getColumn().equals("delete_time");
      }).eq("id", id)).eq("is_delete", 0)).last("limit 1"));
      Assert.notNull(systemAuthDept, "部门已不存在!", new Object[0]);
      SystemAuthDeptVo vo = new SystemAuthDeptVo();
      BeanUtils.copyProperties(systemAuthDept, vo);
      vo.setCreateTime(TimeUtils.timestampToDate(systemAuthDept.getCreateTime()));
      vo.setUpdateTime(TimeUtils.timestampToDate(systemAuthDept.getUpdateTime()));
      return vo;
   }

   public void add(SystemDeptCreateValidate createValidate) {
      SystemAuthDept model;
      if (createValidate.getPid().equals(0)) {
         model = (SystemAuthDept)this.systemAuthDeptMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,pid,name"}).eq("pid", 0)).eq("is_delete", 0)).last("limit 1"));
         Assert.isNull(model, "顶级部门只允许有一个", new Object[0]);
      }

      model = new SystemAuthDept();
      model.setPid(createValidate.getPid());
      model.setName(createValidate.getName());
      model.setDuty(createValidate.getDuty());
      model.setMobile(createValidate.getMobile());
      model.setSort(createValidate.getSort());
      model.setIsStop(createValidate.getIsStop());
      model.setCreateTime(System.currentTimeMillis() / 1000L);
      model.setUpdateTime(System.currentTimeMillis() / 1000L);
      this.systemAuthDeptMapper.insert(model);
   }

   public void edit(SystemDeptUpdateValidate updateValidate) {
      SystemAuthDept model = (SystemAuthDept)this.systemAuthDeptMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(SystemAuthDept.class, (info) -> {
         return !info.getColumn().equals("is_delete") && !info.getColumn().equals("delete_time");
      }).eq("id", updateValidate.getId())).eq("is_delete", 0)).last("limit 1"));
      Assert.notNull(model, "部门不存在", new Object[0]);
      Assert.isFalse(model.getPid().equals(0) && updateValidate.getPid() > 0, "顶级部门不能修改上级", new Object[0]);
      Assert.isFalse(updateValidate.getId().equals(updateValidate.getPid()), "上级部门不能是自己", new Object[0]);
      model.setPid(updateValidate.getPid());
      model.setName(updateValidate.getName());
      model.setDuty(updateValidate.getDuty());
      model.setMobile(updateValidate.getMobile());
      model.setSort(updateValidate.getSort());
      model.setIsStop(updateValidate.getIsStop());
      model.setUpdateTime(System.currentTimeMillis() / 1000L);
      this.systemAuthDeptMapper.updateById(model);
   }

   public void del(Integer id) {
      SystemAuthDept model = (SystemAuthDept)this.systemAuthDeptMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,pid,name"}).eq("id", id)).eq("is_delete", 0)).last("limit 1"));
      Assert.notNull(model, "部门不存在", new Object[0]);
      Assert.isFalse(model.getPid() == 0, "顶级部门不能删除", new Object[0]);
      SystemAuthDept pModel = (SystemAuthDept)this.systemAuthDeptMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,pid,name"}).eq("pid", id)).eq("is_delete", 0)).last("limit 1"));
      Assert.isNull(pModel, "请先删除子级部门", new Object[0]);
      SystemAuthAdmin systemAuthAdmin = (SystemAuthAdmin)this.systemAuthAdminMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,nickname"}).eq("dept_id", id)).eq("is_delete", 0)).last("limit 1"));
      Assert.isNull(systemAuthAdmin, "该部门已被管理员使用,请先移除", new Object[0]);
      model.setIsDelete(1);
      model.setDeleteTime(System.currentTimeMillis() / 1000L);
      this.systemAuthDeptMapper.updateById(model);
   }
}
